@@ -380,11 +380,17 @@ class Router(object):
         try: assert arguments <= 2
         except: raise TypeError('ONCE function received more than 2 arguments')
 
+        def closure(func):
+            async def hidden(r):
+                if iscoroutinefunction(func): return await func(self)
+                else: return func(self)
+            return hidden
+
         if arguments == 1:
             first = args[0]
             try: assert isinstance(first, Callable)
             except AssertionError: raise TypeError(error_message)
-            self.initializers.append(first)
+            self.initializers.append(closure(first))
         else:
             first, second = args
 
@@ -394,8 +400,8 @@ class Router(object):
             try: assert isinstance(second, Callable)
             except AssertionError: raise TypeError(error_message)
 
-            if first == 'startup': self.initializers.append(second)
-            else: self.deinitializers.append(second)
+            if first == 'startup': self.initializers.append(closure(second))
+            else: self.deinitializers.append(closure(second))
 
     async def _register(self):
         i = len(self.initializers)
